@@ -11,16 +11,15 @@
 
 namespace Predis\Connection;
 
-use Predis\ResponseError;
-use Predis\ResponseQueued;
 use Predis\Command\CommandInterface;
-use Predis\Iterator\MultiBulkResponseSimple;
+use Predis\Response\Error as ErrorResponse;
+use Predis\Response\Status as StatusResponse;
 
 /**
  * Standard connection to Redis servers implemented on top of PHP's streams.
- * The connection parameters supported by this class are:
+ * The connection parameters supported by this class are:.
  *
- *  - scheme: it can be either 'tcp' or 'unix'.
+ *  - scheme: it can be either 'redis', 'tcp' or 'unix'.
  *  - host: hostname or IP address of the server.
  *  - port: TCP port of the server.
  *  - path: path of a UNIX domain socket when scheme is 'unix'.
@@ -29,34 +28,23 @@ use Predis\Iterator\MultiBulkResponseSimple;
  *  - async_connect: performs the connection asynchronously.
  *  - tcp_nodelay: enables or disables Nagle's algorithm for coalescing.
  *  - persistent: the connection is left intact after a GC collection.
- *  - iterable_multibulk: multibulk replies treated as iterable objects.
  *
  * @author Daniele Alessandri <suppakilla@gmail.com>
  */
 class StreamConnection extends AbstractConnection
 {
-    private $mbiterable;
-
     /**
-     * {@inheritdoc}
-     */
-    public function __construct(ConnectionParametersInterface $parameters)
-    {
-        $this->mbiterable = (bool) $parameters->iterable_multibulk;
-
-        parent::__construct($parameters);
-    }
-
-    /**
-     * Disconnects from the server and destroys the underlying resource when
-     * PHP's garbage collector kicks in only if the connection has not been
-     * marked as persistent.
+     * Disconnects from the server and destroys the underlying resource when the
+     * garbage collector kicks in only if the connection has not been marked as
+     * persistent.
      */
     public function __destruct()
     {
-        if (isset($this->parameters) && !$this->parameters->persistent) {
-            $this->disconnect();
+        if (isset($this->parameters->persistent) && $this->parameters->persistent) {
+            return;
         }
+
+        $this->disconnect();
     }
 
     /**
@@ -64,47 +52,60 @@ class StreamConnection extends AbstractConnection
      */
     protected function createResource()
     {
-        $parameters = $this->parameters;
-        $initializer = "{$parameters->scheme}StreamInitializer";
+        switch ($this->parameters->scheme) {
+            case 'tcp':
+            case 'redis':
+                return $this->tcpStreamInitializer($this->parameters);
 
-        return $this->$initializer($parameters);
+            case 'unix':
+                return $this->unixStreamInitializer($this->parameters);
+
+            default:
+                throw new \InvalidArgumentException("Invalid scheme: '{$this->parameters->scheme}'.");
+        }
     }
 
     /**
      * Initializes a TCP stream resource.
      *
-     * @param  ConnectionParametersInterface $parameters Parameters used to initialize the connection.
+     * @param ParametersInterface $parameters Initialization parameters for the connection.
+     *
      * @return resource
      */
-    private function tcpStreamInitializer(ConnectionParametersInterface $parameters)
+    protected function tcpStreamInitializer(ParametersInterface $parameters)
     {
-        $uri = "tcp://{$parameters->host}:{$parameters->port}";
+        if (!filter_var($parameters->host, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)) {
+            $uri = "tcp://$parameters->host:$parameters->port";
+        } else {
+            $uri = "tcp://[$parameters->host]:$parameters->port";
+        }
+
         $flags = STREAM_CLIENT_CONNECT;
 
-        if (isset($parameters->async_connect) && $parameters->async_connect) {
+        if (isset($parameters->async_connect) && (bool) $parameters->async_connect) {
             $flags |= STREAM_CLIENT_ASYNC_CONNECT;
         }
 
-        if (isset($parameters->persistent) && $parameters->persistent) {
+        if (isset($parameters->persistent) && (bool) $parameters->persistent) {
             $flags |= STREAM_CLIENT_PERSISTENT;
             $uri .= strpos($path = $parameters->path, '/') === 0 ? $path : "/$path";
         }
 
-        $resource = @stream_socket_client($uri, $errno, $errstr, $parameters->timeout, $flags);
+        $resource = @stream_socket_client($uri, $errno, $errstr, (float) $parameters->timeout, $flags);
 
         if (!$resource) {
             $this->onConnectionError(trim($errstr), $errno);
         }
 
         if (isset($parameters->read_write_timeout)) {
-            $rwtimeout = $parameters->read_write_timeout;
+            $rwtimeout = (float) $parameters->read_write_timeout;
             $rwtimeout = $rwtimeout > 0 ? $rwtimeout : -1;
-            $timeoutSeconds  = floor($rwtimeout);
+            $timeoutSeconds = floor($rwtimeout);
             $timeoutUSeconds = ($rwtimeout - $timeoutSeconds) * 1000000;
             stream_set_timeout($resource, $timeoutSeconds, $timeoutUSeconds);
         }
 
-        if (isset($parameters->tcp_nodelay) && version_compare(PHP_VERSION, '5.4.0') >= 0) {
+        if (isset($parameters->tcp_nodelay) && function_exists('socket_import_stream')) {
             $socket = socket_import_stream($resource);
             socket_set_option($socket, SOL_TCP, TCP_NODELAY, (int) $parameters->tcp_nodelay);
         }
@@ -115,24 +116,37 @@ class StreamConnection extends AbstractConnection
     /**
      * Initializes a UNIX stream resource.
      *
-     * @param  ConnectionParametersInterface $parameters Parameters used to initialize the connection.
+     * @param ParametersInterface $parameters Initialization parameters for the connection.
+     *
      * @return resource
      */
-    private function unixStreamInitializer(ConnectionParametersInterface $parameters)
+    protected function unixStreamInitializer(ParametersInterface $parameters)
     {
+        if (!isset($parameters->path)) {
+            throw new InvalidArgumentException('Missing UNIX domain socket path.');
+        }
+
         $uri = "unix://{$parameters->path}";
         $flags = STREAM_CLIENT_CONNECT;
 
-        if ($parameters->persistent) {
+        if ((bool) $parameters->persistent) {
             $flags |= STREAM_CLIENT_PERSISTENT;
         }
 
-        $resource = @stream_socket_client($uri, $errno, $errstr, $parameters->timeout, $flags);
+        $resource = @stream_socket_client($uri, $errno, $errstr, (float) $parameters->timeout, $flags);
 
         if (!$resource) {
             $this->onConnectionError(trim($errstr), $errno);
         }
 
+        if (isset($parameters->read_write_timeout)) {
+            $rwtimeout = (float) $parameters->read_write_timeout;
+            $rwtimeout = $rwtimeout > 0 ? $rwtimeout : -1;
+            $timeoutSeconds = floor($rwtimeout);
+            $timeoutUSeconds = ($rwtimeout - $timeoutSeconds) * 1000000;
+            stream_set_timeout($resource, $timeoutSeconds, $timeoutUSeconds);
+        }
+
         return $resource;
     }
 
@@ -141,10 +155,10 @@ class StreamConnection extends AbstractConnection
      */
     public function connect()
     {
-        parent::connect();
-
-        if ($this->initCmds) {
-            $this->sendInitializationCommands();
+        if (parent::connect() && $this->initCommands) {
+            foreach ($this->initCommands as $command) {
+                $this->executeCommand($command);
+            }
         }
     }
 
@@ -160,36 +174,24 @@ class StreamConnection extends AbstractConnection
     }
 
     /**
-     * Sends the initialization commands to Redis when the connection is opened.
-     */
-    private function sendInitializationCommands()
-    {
-        foreach ($this->initCmds as $command) {
-            $this->writeCommand($command);
-        }
-        foreach ($this->initCmds as $command) {
-            $this->readResponse($command);
-        }
-    }
-
-    /**
-     * Performs a write operation on the stream of the buffer containing a
+     * Performs a write operation over the stream of the buffer containing a
      * command serialized with the Redis wire protocol.
      *
-     * @param string $buffer Redis wire protocol representation of a command.
+     * @param string $buffer Representation of a command in the Redis wire protocol.
      */
-    protected function writeBytes($buffer)
+    protected function write($buffer)
     {
         $socket = $this->getResource();
 
         while (($length = strlen($buffer)) > 0) {
-            $written = fwrite($socket, $buffer);
+            $written = @fwrite($socket, $buffer);
 
             if ($length === $written) {
                 return;
             }
+
             if ($written === false || $written === 0) {
-                $this->onConnectionError('Error while writing bytes to the server');
+                $this->onConnectionError('Error while writing bytes to the server.');
             }
 
             $buffer = substr($buffer, $written);
@@ -202,32 +204,24 @@ class StreamConnection extends AbstractConnection
     public function read()
     {
         $socket = $this->getResource();
-        $chunk  = fgets($socket);
+        $chunk = fgets($socket);
 
         if ($chunk === false || $chunk === '') {
-            $this->onConnectionError('Error while reading line from the server');
+            $this->onConnectionError('Error while reading line from the server.');
         }
 
-        $prefix  = $chunk[0];
+        $prefix = $chunk[0];
         $payload = substr($chunk, 1, -2);
 
         switch ($prefix) {
             case '+':
-                switch ($payload) {
-                    case 'OK':
-                        return true;
-
-                    case 'QUEUED':
-                        return new ResponseQueued();
-
-                    default:
-                        return $payload;
-                }
+                return StatusResponse::get($payload);
 
             case '$':
                 $size = (int) $payload;
+
                 if ($size === -1) {
-                    return null;
+                    return;
                 }
 
                 $bulkData = '';
@@ -237,7 +231,7 @@ class StreamConnection extends AbstractConnection
                     $chunk = fread($socket, min($bytesLeft, 4096));
 
                     if ($chunk === false || $chunk === '') {
-                        $this->onConnectionError('Error while reading bytes from the server');
+                        $this->onConnectionError('Error while reading bytes from the server.');
                     }
 
                     $bulkData .= $chunk;
@@ -250,15 +244,12 @@ class StreamConnection extends AbstractConnection
                 $count = (int) $payload;
 
                 if ($count === -1) {
-                    return null;
-                }
-                if ($this->mbiterable) {
-                    return new MultiBulkResponseSimple($this, $count);
+                    return;
                 }
 
                 $multibulk = array();
 
-                for ($i = 0; $i < $count; $i++) {
+                for ($i = 0; $i < $count; ++$i) {
                     $multibulk[$i] = $this->read();
                 }
 
@@ -268,40 +259,34 @@ class StreamConnection extends AbstractConnection
                 return (int) $payload;
 
             case '-':
-                return new ResponseError($payload);
+                return new ErrorResponse($payload);
 
             default:
-                $this->onProtocolError("Unknown prefix: '$prefix'");
+                $this->onProtocolError("Unknown response prefix: '$prefix'.");
+
+                return;
         }
     }
 
     /**
      * {@inheritdoc}
      */
-    public function writeCommand(CommandInterface $command)
+    public function writeRequest(CommandInterface $command)
     {
-        $commandId = $command->getId();
+        $commandID = $command->getId();
         $arguments = $command->getArguments();
 
-        $cmdlen = strlen($commandId);
+        $cmdlen = strlen($commandID);
         $reqlen = count($arguments) + 1;
 
-        $buffer = "*{$reqlen}\r\n\${$cmdlen}\r\n{$commandId}\r\n";
+        $buffer = "*{$reqlen}\r\n\${$cmdlen}\r\n{$commandID}\r\n";
 
-        for ($i = 0, $reqlen--; $i < $reqlen; $i++) {
+        for ($i = 0, $reqlen--; $i < $reqlen; ++$i) {
             $argument = $arguments[$i];
             $arglen = strlen($argument);
             $buffer .= "\${$arglen}\r\n{$argument}\r\n";
         }
 
-        $this->writeBytes($buffer);
-    }
-
-    /**
-     * {@inheritdoc}
-     */
-    public function __sleep()
-    {
-        return array_merge(parent::__sleep(), array('mbiterable'));
+        $this->write($buffer);
     }
 }
